@@ -5,26 +5,47 @@ import uuid as _uu
 # Define our alphabet.
 _ALPHABET = "23456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz"
 
-def uuid(url=None):
+def encode(uuid):
+    """
+    Encodes a UUID into a string (LSB first) according to the alphabet
+    If leftmost (MSB) bits 0, string might be shorter
+    """
+    unique_id = uuid.int
+    alphabet_length = len(_ALPHABET)
+    output = ""
+    while unique_id > 0:
+        digit = unique_id % alphabet_length
+        output += _ALPHABET[digit]
+        unique_id = int(unique_id / alphabet_length)
+    return output
+
+def decode(string):
+    """
+    Decodes a string according to the current alphabet into a UUID
+    Raises ValueError when encountering illegal characters or too long string
+    If string too short, fills leftmost (MSB) bits with 0.
+    """
+    number = 0
+    for char in string[::-1]:
+        value = _ALPHABET.index(char)
+        number = number * len(_ALPHABET) + value
+    return _uu.UUID(int = number)
+
+def uuid(name=None):
     """
     Generate and return a UUID.
 
-    If the url parameter is provided, set the namespace to the provided
-    URL and generate a UUID.
+    If the name parameter is provided, set the namespace to the provided
+    name and generate a UUID.
     """
-    # If no URL is given, generate a random UUID.
-    if url is None:
-        unique_id = _uu.uuid4().int
+    # If no name is given, generate a random UUID.
+    if name is None:
+        uuid = _uu.uuid4()
+    elif not "http" in name:
+        uuid = _uu.uuid5(_uu.NAMESPACE_DNS, name)
     else:
-        unique_id = _uu.uuid3(_uu.NAMESPACE_URL, url).int
-
-    alphabet_length = len(_ALPHABET)
-    output = []
-    while unique_id > 0:
-        digit = unique_id % alphabet_length
-        output.append(_ALPHABET[digit])
-        unique_id = int(unique_id / alphabet_length)
-    return "".join(output)
+        uuid = _uu.uuid5(_uu.NAMESPACE_URL, name)
+    return encode(uuid)
 
 def get_alphabet():
     """Return the current alphabet used for new UUIDs."""
@@ -46,4 +67,3 @@ def set_alphabet(alphabet):
         _ALPHABET = new_alphabet
     else:
         raise ValueError("Alphabet with more than one unique symbols required.")
-
